@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/budb/server.c,v 1.6.2.2 2001/10/13 04:21:07 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/budb/server.c,v 1.6.2.3 2002/04/21 04:14:53 zacheiss Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -472,9 +472,8 @@ main(argc, argv)
 							  afsconf_GetKey,
 							  (char *) 0);
 
-    /* These two lines disallow jumbograms */
-    rx_maxReceiveSize = OLD_MAX_PACKET_SIZE;
-    rxi_nSendFrags = rxi_nRecvFrags = 1;
+    /* Disable jumbograms */
+    rx_SetNoJumbo();
 
     tservice = rx_NewService(0, BUDB_SERVICE, "BackupDatabase",
 			     sca, 3, BUDB_ExecuteRequest);
